from sqlalchemy import func

from models import Account
from database import DatabaseManager
from database.exceptions import OutOfAccountSpaceError, NonexistentAccountError
from utils.constants import MIN_ACCOUNT_NUMBER, MAX_ACCOUNT_NUMBER


def get_next_id() -> int:
    with DatabaseManager.get_session() as session:
        new_id = session.query(func.max(Account.account_number)).scalar()
        new_id = new_id if new_id is not None else MIN_ACCOUNT_NUMBER

        if new_id > MAX_ACCOUNT_NUMBER:
            raise OutOfAccountSpaceError("Too many users already exist, cannot open new account")

        return new_id


def create_account() -> int:
    new_id = get_next_id()

    with DatabaseManager.get_session() as session:
        new_account = Account(account_number=new_id, balance=0)
        session.add(new_account)
        session.commit()
    return new_id


def get_account_balance(account_number: int) -> int:
    with DatabaseManager.get_session() as session:
        account: Account = session.query(Account).where(Account.account_number == account_number).one_or_none()
        if account is NotImplemented:
            raise NonexistentAccountError(f"Account with number {account_number} doesn't exist")

        return account.balance

def withdraw_from_account():
    pass


def deposit_into_account():
    pass


def delete_account():
    pass
